"""
Choice sets used by the models, views and forms.
"""

from utilities.choices import ChoiceSet

__all__ = (
    "BridgeDomainL2UUCChoices",
    "ContractScopeChoices",
    "ContractQoSClassChoices",
    "ContractTargetDSCPChoices",
    "ContractFilterACtionChoices",
    "ContractFilterEntryEtherTypeChoices",
    "ContractFilterDirectivesChoices",
    "ContractFilterEntryARPFlagChoices",
    "ContractFilterEntryIPProtocolChoices",
    "DomainTypeChoices",
    "PoolAllocationModeChoices",
    "AAEPStaticBindingModeChoices",
    "LinkLevelNegotiationChoices",
    "LinkLevelSpeedChoices",
)


class BridgeDomainL2UUCChoices(ChoiceSet):
    """
    Defines the choices availble for the L2 Unknown Unicast.
    """

    FLOOD = "flood"
    HARDWARE_PROXY = "hardware_proxy"

    CHOICES = [
        ('', "----"),
        (FLOOD, "flood"),
        (HARDWARE_PROXY, "hardware_proxy"),
    ]


class ContractScopeChoices(ChoiceSet):
    """
    Defines the choices availble for the Contract scope.
    """

    SCOPE_VRF = "vrf"
    SCOPE_TENANT = "tenant"
    SCOPE_GLOBAL = "global"

    CHOICES = [
        ('', "----"),
        (SCOPE_VRF, "vrf"),
        (SCOPE_TENANT, "tenant"),
        (SCOPE_GLOBAL, "global"),
    ]


class ContractQoSClassChoices(ChoiceSet):
    """
    Defines the choices availble for the Contract Qos Class.
    """

    LEVEL1 = "level1"
    LEVEL2 = "level2"
    LEVEL3 = "level3"
    LEVEL4 = "level4"
    LEVEL5 = "level5"
    LEVEL6 = "level6"

    CHOICES = [
        ('', "----"),
        (LEVEL1, "level1"),
        (LEVEL2, "level2"),
        (LEVEL3, "level3"),
        (LEVEL4, "level4"),
        (LEVEL5, "level5"),
        (LEVEL6, "level6"),
    ]


class ContractTargetDSCPChoices(ChoiceSet):
    """
    Defines the choices availble for the Contract Target DSCP.
    """

    AF11 = "af11"
    AF12 = "af12"
    AF13 = "af13"
    AF21 = "af21"
    AF22 = "af22"
    AF23 = "af23"
    AF31 = "af31"
    AF32 = "af32"
    AF33 = "af33"
    AF41 = "af41"
    AF42 = "af42"
    AF43 = "af43"
    CS0 = "cs0"
    CS1 = "cs1"
    CS2 = "cs2"
    CS3 = "cs3"
    CS4 = "cs4"
    CS5 = "cs5"
    CS6 = "cs6"
    CS7 = "cs7"
    EXPEDITED_FORWARDING = "expedited_forwarding"
    VOICE_ADMIT = "voice_admit"

    CHOICES = [
        ('', "----"),
        (AF11, "af11"),
        (AF12, "af12"),
        (AF13, "af13"),
        (AF21, "af21"),
        (AF22, "af22"),
        (AF23, "af23"),
        (AF31, "af31"),
        (AF32, "af32"),
        (AF33, "af33"),
        (AF41, "af41"),
        (AF42, "af42"),
        (AF43, "af43"),
        (CS0, "cs0"),
        (CS1, "cs1"),
        (CS2, "cs2"),
        (CS3, "cs3"),
        (CS4, "cs4"),
        (CS5, "cs5"),
        (CS6, "cs6"),
        (CS7, "cs7"),
        (EXPEDITED_FORWARDING, "expedited_forwarding"),
        (VOICE_ADMIT, "voice_admit"),
    ]


class ContractFilterDirectivesChoices(ChoiceSet):
    """
    Defines the choices availble for the Contract Filter Directives.
    """

    LOG = "log"
    ENABLE_POLICY_COMPRESSION = "enable_policy_compression"

    CHOICES = [
        (LOG, "log"),
        (ENABLE_POLICY_COMPRESSION, "enable_policy_compression"),
    ]


class ContractFilterACtionChoices(ChoiceSet):
    """
    Defines the choices availble for the Contract Filter Action.
    """

    PERMIT = "permit"
    DENY = "deny"

    CHOICES = [
        ('', "----"),
        (PERMIT, "permit"),
        (DENY, "deny"),
    ]


class ContractFilterEntryEtherTypeChoices(ChoiceSet):
    """
    Defines the choices availble for the Contract Filter Entry EtherType.
    """

    IP = "ip"
    IPv4 = "ipv4"
    IPv6 = "ipv6"
    MPLS_UNICAST = "mpls_unicast"
    UNSPECIFIED = "unspecified"
    TRILL = "trill"
    ARP = "arp"
    FCOE = "fcoe"
    MAC_SECURITY = "mac_security"

    CHOICES = [
        ('', "----"),
        (IP, "ip"),
        (IPv4, "ipv4"),
        (IPv6, "ipv6"),
        (MPLS_UNICAST, "mpls_unicast"),
        (UNSPECIFIED, "unspecified"),
        (TRILL, "trill"),
        (ARP, "arp"),
        (FCOE, "fcoe"),
        (MAC_SECURITY, "mac_security"),
    ]


class ContractFilterEntryIPProtocolChoices(ChoiceSet):
    """
    Defines the choices availble for the Contract Filter Entry IP Protocol.
    """

    EIGRP = "eigrp"
    EGP = "egp"
    ICMP = "icmp"
    ICMPv6 = "icmpv6"
    IGMP = "igmp"
    IGP = "igp"
    L2TP = "l2tp"
    OSPF = "ospf"
    PIM = "pim"
    TCP = "tcp"
    UDP = "udp"
    UNSPECIFIED = "unspecified"

    CHOICES = [
        ('', "----"),
        (EIGRP, "eigrp"),
        (EGP, "egp"),
        (ICMP, "icmp"),
        (ICMPv6, "icmpv6"),
        (IGMP, "igmp"),
        (IGP, "igp"),
        (L2TP, "l2tp"),
        (OSPF, "ospf"),
        (PIM, "pim"),
        (TCP, "tcp"),
        (UDP, "udp"),
        (UNSPECIFIED, "unspecified"),
    ]


class ContractFilterEntryARPFlagChoices(ChoiceSet):
    """
    Defines the choices availble for the Contract Filter Entry ARP Flag.
    """

    ARP_REPLY = "reply"
    ARP_REQUEST = "request"
    UNSPECIFIED = "unspecified"

    CHOICES = [
        ('', "----"),
        (ARP_REPLY, "reply"),
        (ARP_REQUEST, "request"),
        (UNSPECIFIED, "unspecified"),
    ]


class DomainTypeChoices(ChoiceSet):
    """
    Defines the choices availble for the Domain Type.
    """

    PHYSICAL = "physical"
    VMM = "vmm"
    L3 = "l3"

    CHOICES = [
        ('', "----"),
        (PHYSICAL, "physical"),
        (VMM, "vmm"),
        (L3, "l3"),
    ]


class PoolAllocationModeChoices(ChoiceSet):
    """
    Defines the choices availble for the Pool Allocation Mode.
    """

    STATIC = "static"
    DYNAMIC = "dynamic"

    CHOICES = [
        ('', "----"),
        (STATIC, "static"),
        (DYNAMIC, "dynamic"),
    ]


class AAEPStaticBindingModeChoices(ChoiceSet):
    """
    Defines the choices availble for the static binding within an AAEP.
    """

    TRUNK = "trunk"
    ACCESS8021p = "access_8021p"
    ACCESSuntagged = "access_untagged"

    CHOICES = [
        ('', "----"),
        (TRUNK, "trunk"),
        (ACCESS8021p, "access_8021p"),
        (ACCESSuntagged, "access_untagged"),
    ]


class LinkLevelNegotiationChoices(ChoiceSet):
    """
    Defines the choices availble for an interface link level negotiation.
    """

    OFF = "off"
    ON = "on"
    ENFORCE = "enforce"

    CHOICES = [
        ('', "----"),
        (OFF, "off"),
        (ON, "on"),
        (ENFORCE, "enforce"),
    ]


class LinkLevelSpeedChoices(ChoiceSet):
    """
    Defines the choices availble for an interface link level speed.
    """

    INHERIT = "inherit"
    HUNDRED_MBPS = "100mbps"
    ONE_GBPS = "1gbps"
    TEN_GBPS = "10gbps"
    TWENTYFIVE_GBPS = "25gbps"
    FORTY_GBPS = "40gbps"
    HUNDRED_GBPS = "100gbps"
    FOURHUNDRET_GBPS = "400gbps"

    CHOICES = [
        ('', "----"),
        (INHERIT, "inherit"),
        (HUNDRED_MBPS, "100mbps"),
        (ONE_GBPS, "1gbps"),
        (TEN_GBPS, "10gbps"),
        (TWENTYFIVE_GBPS, "25gbps"),
        (FORTY_GBPS, "40gbps"),
        (HUNDRED_GBPS, "100gbps"),
        (FOURHUNDRET_GBPS, "400gbps"),
    ]



"""
Define the django form elements for the user interface. 
"""

from netbox.forms import NetBoxModelForm
from utilities.forms.fields import CommentField, SlugField
from utilities.forms.rendering import FieldSet, TabbedGroups
from .. models import contract_model

class ContractForm(NetBoxModelForm):

    slug = SlugField()
    comments = CommentField()

    fieldsets = (
        FieldSet(
            TabbedGroups(
                FieldSet(
                    'name',
                    'slug',
                    'description',
                    'scope',
                    'qos_class',
                    'target_dscp',
                    name = 'Contract',
                ),
                FieldSet(
                    'vrfs_consume',
                    'vrfs_provide',
                    name = 'vzAny',
                ),
            ),
        ),
    )

    class Meta:
        model = contract_model.Contract
        fields = (
            'name',
            'slug',
            'description',
            'comments',
            'scope',
            'qos_class',
            'target_dscp',
            'vrfs_consume',
            'vrfs_provide',
        )



"""
Define the logic of the plugin.
"""

from django.shortcuts import render
from django.views.generic import View
from dcim.models import device_components
from netbox.views import generic
from utilities.views import GetRelatedModelsMixin, ViewTab, register_model_view

from .. import tables
from .. models import ipg_model
from .. forms import ipg_form
from .. import filtersets

__all__ = (
    "PolicyGroupView",
    "PolicyGroupListView",
    "PolicyGroupEditView",
    "PolicyGroupDeleteView",
    "PolicyGroupBulkDeleteView",
    "PolicyGroupAssignementView",
    "PolicyGroupAssignementListView",
    "PolicyGroupAssignementEditView",
    "PolicyGroupAssignementDeleteView",
    "PolicyGroupAssignementBulkDeleteView",
    "PolicyGroupDcimInterfaceTabView",
)


class PolicyGroupView(GetRelatedModelsMixin, generic.ObjectView):
    queryset = ipg_model.PolicyGroup.objects.all()

    def get_extra_context(self, request, instance):

        ipgassignement_table = tables.IPGAssignementTable(instance.ipgassignement.all())

        return {
            'ipgassignement_table': ipgassignement_table,
            'related_models': self.get_related_models(request, instance),
        }


class PolicyGroupListView(generic.ObjectListView):
    queryset = ipg_model.PolicyGroup.objects.all()
    table = tables.IPGTable
    filterset = filtersets.PolicyGroupListFilterSet
    filterset_form = ipg_form.PolicyGroupFilterForm

class PolicyGroupEditView(generic.ObjectEditView):
    queryset = ipg_model.PolicyGroup.objects.all()
    form = ipg_form.PolicyGroupForm
    default_return_url = 'plugins:netbox_aci:policygroup_list'


class PolicyGroupDeleteView(generic.ObjectDeleteView):
    queryset = ipg_model.PolicyGroup.objects.all()


class PolicyGroupBulkDeleteView(generic.BulkDeleteView):
    queryset = ipg_model.PolicyGroup.objects.all()
    table = tables.IPGTable


class PolicyGroupAssignementView(GetRelatedModelsMixin, generic.ObjectView):
    queryset = ipg_model.PolicyGroupAssignement.objects.all()

    def get_extra_context(self, request, instance):

        return {
            'related_models': self.get_related_models(request, instance),

        }


class PolicyGroupAssignementListView(generic.ObjectListView):
    queryset = ipg_model.PolicyGroupAssignement.objects.all()
    table = tables.IPGAssignementTable


class PolicyGroupAssignementEditView(generic.ObjectEditView):
    queryset = ipg_model.PolicyGroupAssignement.objects.all()
    form = ipg_form.PolicyGroupAssignementForm
    default_return_url = 'plugins:netbox_aci:policygroupassignement_list'


class PolicyGroupAssignementDeleteView(generic.ObjectDeleteView):
    queryset = ipg_model.PolicyGroupAssignement.objects.all()


class PolicyGroupAssignementBulkDeleteView(generic.BulkDeleteView):
    queryset = ipg_model.PolicyGroupAssignement.objects.all()
    table = tables.IPGAssignementTable


@register_model_view(device_components.Interface, "ipg")
class PolicyGroupDcimInterfaceTabView(View):
    tab = ViewTab(
        label="ACI IPG Assignements",
        badge=lambda obj: ipg_model.PolicyGroupAssignement.objects.filter(
            interface=obj
        ).count(),
        hide_if_empty = True,
        permission="netbox_aci.view_policygroup",
    )

    def get(self, request, pk):
        intf = device_components.Interface.objects.get(pk=pk)
        policy_group = ipg_model.PolicyGroupAssignement.objects.filter(interface=intf)
        ipg = ipg_model.PolicyGroup.objects.filter(ipgassignement__in=policy_group)

        return render(
            request,
            "netbox_aci/policygroup_tab.html",
            context={
                "tab": self.tab,
                "object": intf,
                "ipg_table": ipg,
            },
        )



"""
Import models scripts
"""

from . ap_view import *
from . epg_view import *
from . contract_view import *
from . contract_subject_view import *
from . contract_filter_view import *
from . contract_filter_entry_view import *
from . bd_view import *
from . l3out_view import *
from . esg_view import *
from . domain_view import *
from . aaep_view import *
from . ipg_view import *
from . policy_link_level_view import *
